package de.hft.stuttgart.citydoctor2.datastructure;

import de.hft.stuttgart.citydoctor2.check.Check;
import de.hft.stuttgart.citydoctor2.check.CheckError;
import de.hft.stuttgart.citydoctor2.check.CheckId;
import de.hft.stuttgart.citydoctor2.math.TransformationMatrix;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.citygml4j.core.model.core.ImplicitGeometry;

import java.io.Serial;
import java.util.List;

/**
 * Datastructure for representation and resolving of implicit geometries
 *
 * @author Riegel
 */
public class ImplicitGeometryHolder extends Geometry {

    private static final Logger logger = LogManager.getLogger(ImplicitGeometryHolder.class);
    @Serial
    private static final long serialVersionUID = -8938931081577196349L;

    private ImplicitGeometry cgmlImplicitGeometry = null;

    private final PrototypeGeometryType type;
    private final Geometry prototypeGeometry;

    public ImplicitGeometryHolder(ImplicitGeometry cgmlImplicitGeometry, LibraryObject libObject) {
        super(libObject.getType(), libObject.getLod());
        prototypeGeometry = libObject;
        type = PrototypeGeometryType.LIBRARY_OBJECT;
        this.cgmlImplicitGeometry = cgmlImplicitGeometry;
        applyTransformation();
    }

    public ImplicitGeometryHolder(ImplicitGeometry cgmlImplicitGeometry, RelativeGeometry relativeGeometry) {
        super(relativeGeometry.getType(), relativeGeometry.getLod());
        prototypeGeometry = relativeGeometry;
        type = PrototypeGeometryType.RELATIVE_GEOMETRY;
        this.cgmlImplicitGeometry = cgmlImplicitGeometry;
        applyTransformation();
    }


    /**
     * Applies the transformation matrix of the implicit geometry to a copy of the reference geometry and copies the resulting
     * transformed geometry into this object.
     */
    private void applyTransformation() {
        TransformationMatrix transformMatrix = new TransformationMatrix(cgmlImplicitGeometry);
        Geometry transformedGeom = transformMatrix.transformGeometry(prototypeGeometry);
        transformedGeom.getPolygons().forEach(this::addPolygon);
        this.updateEdgesAndVertices();
    }

    public PrototypeGeometryType getPrototypeGeometryType() {
        return type;
    }

    @Override
    public void accept(Check c) {
        prototypeGeometry.accept(c);
    }

    @Override
    public boolean containsError(CheckId checkIdentifier) {
        return prototypeGeometry.containsError(checkIdentifier);
    }

    @Override
    public void clearAllContainedCheckResults() {
        prototypeGeometry.clearAllContainedCheckResults();
    }

    @Override
    public void collectContainedErrors(List<CheckError> errors) {
        prototypeGeometry.clearAllContainedCheckResults();
    }

    @Override
    public boolean containsAnyError() {
        return prototypeGeometry.containsAnyError();
    }

    @Override
    public void prepareForChecking() {
        this.updateEdgesAndVertices();
        prototypeGeometry.prepareForChecking();
    }

    @Override
    public boolean isValidated() {
        return prototypeGeometry.isValidated();
    }

    public enum PrototypeGeometryType {
        LIBRARY_OBJECT, RELATIVE_GEOMETRY
    }

}
