package de.hft.stuttgart.citydoctor2.zip;

import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.utils.ArchivePacker;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;

import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

public class CityGmlZipArchive implements Serializable {

    private static final Logger logger = LogManager.getLogger(CityGmlZipArchive.class);

    @Serial
    private static final long serialVersionUID = 2168389511043362615L;

    private List<CityGmlZipEntry> entries;
    private final Path archivePath;
    private ZipFile zipFile;

    public static CityGmlZipArchive fromZipFile(String zipFile, ParserConfiguration config) {
        ArrayList<CityGmlZipEntry> archiveEntries = new ArrayList<>();
        ZipFile zip = null;
        try (ZipInputStream zis = new ZipInputStream(new FileInputStream(zipFile))) {
            zip = new ZipFile(zipFile);
            ZipEntry ze;
            while ((ze = zis.getNextEntry()) != null )
            {
                if (ze.isDirectory()) {
                    continue;
                }
                if (ze.getName().endsWith(".gml")) {
                    archiveEntries.add(CityGmlZipEntry.of(ze, zip, config));
                }
            }
        } catch (IOException e) {
            logger.error(e);
        } finally {
            if(zip != null){
                try {
                    zip.close();
                } catch (IOException e) {
                    logger.error(e);
                }
            }
        }
        logger.info("Zip-archive successfully uncompressed, {} CityGml files detected", archiveEntries.size());
        return new CityGmlZipArchive(archiveEntries, Path.of(zipFile));
    }

    public static CityGmlZipArchive register(String zipFile) {
        ArrayList<CityGmlZipEntry> archiveEntries = new ArrayList<>();
        CityGmlZipArchive cgmlArchive = new CityGmlZipArchive(Path.of(zipFile));
        try (ZipInputStream zis = new ZipInputStream(new FileInputStream(zipFile))) {
            ZipEntry ze;
            while ((ze = zis.getNextEntry()) != null )
            {
                if (ze.isDirectory()) {
                    continue;
                }
                if (ze.getName().endsWith(".gml")) {
                    archiveEntries.add(CityGmlZipEntry.register(ze));
                }
            }
            cgmlArchive.setEntries(archiveEntries);
            return cgmlArchive;
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public void mountArchive(ParserConfiguration config){
        try (ZipFile zip = new ZipFile(archivePath.toFile())) {
            zipFile = zip;
            for (CityGmlZipEntry entry : entries){
                entry.loadEntry(zip, config);
            }
        } catch (IOException e) {
            logger.error(e);
        } finally {
            zipFile = null;
        }
    }

    private CityGmlZipArchive(Path archivePath){
        this.archivePath = archivePath;
    }

    private CityGmlZipArchive(List<CityGmlZipEntry> entries, Path archivePath) {
        setEntries(entries);
        this.archivePath = archivePath;
    }

    private void setEntries(List<CityGmlZipEntry> entries) {
        this.entries = entries;
        entries.forEach(e -> e.setArchive(this));
    }

    public void exportToZipFile(String path)  {
        ArchivePacker.packArchive(path, this);
    }


    public InputStream getInputStream(CityGmlZipEntry entry) throws IOException {
        if(zipFile == null){
            throw new ZipException("Requested InputStream from unmounted CityGmlArchive");
        }
        return zipFile.getInputStream(zipFile.getEntry(entry.getFileName()));
    }

    public CityGmlZipEntry getEntry(String fileName) {
        for(CityGmlZipEntry entry : entries){
            if(entry.getFileName().equals(fileName)){
                return entry;
            }
        }
        return null;
    }


    public Path getArchivePath() {
        return archivePath;
    }

    public List<CityGmlZipEntry> getEntries() {
        return entries;
    }
}
