package de.hft.stuttgart.citydoctor2.zip;

import de.hft.stuttgart.citydoctor2.datastructure.CityDoctorModel;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParseException;
import de.hft.stuttgart.citydoctor2.parser.CityGmlParser;
import de.hft.stuttgart.citydoctor2.parser.InvalidGmlFileException;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class CityGmlZipEntry {

    private static final Logger logger = LogManager.getLogger(CityGmlZipEntry.class);

    private final String fileName;
    private CityDoctorModel model;
    private CityGmlZipArchive parentArchive;
    private boolean validated = false;
    private boolean decompressed = false;
    private long size = -1L;
    private static final long MB = 1024 * 1024L;

    public static CityGmlZipEntry of(ZipEntry entry, ZipFile archive, ParserConfiguration config){
        CityGmlZipEntry ze = CityGmlZipEntry.register(entry);
        return ze.loadEntry(archive, config);
    }

    public CityGmlZipEntry loadEntry(ZipFile zip ,ParserConfiguration config){
        if (decompressed){
            return this;
        }
        ZipEntry ze = zip.getEntry(fileName);
        try {
            if (!entrySizeWithinMemoryLimits(ze,zip)) {
                return new ErroneousEntry(ze, ZipEntryErrorType.EXCESSIVE_FILESIZE);
            }
            CityGmlParser.gagLogger(true);
            this.model = CityGmlParser.parseCityGmlZipEntry(ze, zip, config);
            this.decompressed = true;
            return this;
        } catch (CityGmlParseException | InvalidGmlFileException e) {
            logger.error(e);
            return new ErroneousEntry(ze, ZipEntryErrorType.INVALID_CITY_GML_FILE);
        } catch (IOException e){
            logger.error(e);
            return new ErroneousEntry(ze, ZipEntryErrorType.IO_ERROR);
        }
    }

    public static CityGmlZipEntry register(ZipEntry entry){
        return new CityGmlZipEntry(entry, false);
    }

    private boolean entrySizeWithinMemoryLimits(ZipEntry ze, ZipFile zip) throws IOException {
        long memoryLimit = (long) Math.ceil(((double) Runtime.getRuntime().maxMemory() / MB)*0.9);
        if (size != -1L){
            return memoryLimit > size;
        }
        if (ze.getSize() == -1L){
            InputStream is = zip.getInputStream(ze);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            for(int i = is.read(); i != -1; i=is.read()) {
                baos.write(i);
                if ((baos.size() / MB) + 1 > memoryLimit) {
                    //Entry exceeds memory limit
                    return false;
                }
            }
            // end of stream reached without exceeding memory limit
            this.size = baos.size();
            return true;

        } else {
            size = (long) Math.ceil((double) ze.getSize() / MB);
            return memoryLimit > size;
        }
    }

    protected CityGmlZipEntry(ZipEntry entry, boolean decompressed) {
        this.fileName = entry.getName();
        this.model = null;
        this.decompressed = decompressed;
    }


    public void setArchive(CityGmlZipArchive archive){
        parentArchive = archive;
    }

    public CityGmlZipArchive getArchive(){
        return parentArchive;
    }


    public boolean isValidated(){
        return validated;
    }

    public String getFileName() {
        return fileName;
    }

    public CityDoctorModel getModel() {
        return model;
    }

}
